#include <idt/Idt.hpp>
#include <system/System.hpp>

/**
 * IdtMounter's method for mounting the interrupt descriptor table
 */
extern "C" void _mountIdt();

/**
 * The IDT consisting of 256 IDT entries, and the IDT pointer
 */
struct IdtEntry idt[256];
struct IdtPointer idtPointer;

/**
 * Creates an IDT gate
 */
void Idt::createIdtGate(unsigned char index, unsigned int base,
		unsigned short kernelSegment, unsigned char flags) {
	idt[index].baseLow = base & 0xFFFF;
	idt[index].baseHigh = (base >> 16) & 0xFFFF;
	idt[index].kernelSegment = kernelSegment;
	idt[index].zero = 0;
	idt[index].flags = flags;
}

/**
 * Installs the interrupt descriptor table.
 *
 * - Sets the IDT pointers limit to the size of 256 IDT entries
 * - Sets the IDT pointers base to the address of the idt
 * - Fills memory 256 times the size of a IDT entry with zeros starting at the
 * address of the IDT
 */
void Idt::install() {
	idtPointer.limit = (sizeof(struct IdtEntry) * 256) - 1;
	idtPointer.base = reinterpret_cast<unsigned int>(&idt);

	unsigned char* idtp = reinterpret_cast<unsigned char*>(&idt);
	for(int i = 0; i < sizeof(struct IdtEntry) * 256; i++) {
		idtp[i] = 0;
	}

	_mountIdt();
}
