#include <system/System.hpp>
#include <idt/Idt.hpp>
#include <idt/InterruptRequests.hpp>
#include <bios/Video.hpp>


extern "C" void _ireq0();
extern "C" void _ireq1();
extern "C" void _ireq2();
extern "C" void _ireq3();
extern "C" void _ireq4();
extern "C" void _ireq5();
extern "C" void _ireq6();
extern "C" void _ireq7();
extern "C" void _ireq8();
extern "C" void _ireq9();
extern "C" void _ireq10();
extern "C" void _ireq11();
extern "C" void _ireq12();
extern "C" void _ireq13();
extern "C" void _ireq14();
extern "C" void _ireq15();

void *requestRoutines[16] = {
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0
};

extern "C" void _requestHandler(struct regs *r) {
        // Empty function pointer
        void (*handler) (struct regs *r);

        // Call the custom handler if it exists
        handler = reinterpret_cast<void (*)(regs*)>(requestRoutines[r->int_no - 32]);
        if(handler) {
                handler(r);
        }

        if(r->int_no >= 40) {
                outportByte(0xA0, 0x20);
        }

        outportByte(0x20, 0x20);
}

void InterruptRequests::uninstallRequestHandler(int irq) {
        requestRoutines[irq] = 0;
}

void InterruptRequests::installRequestHandler(int irq, void (*handler)(struct regs *r)) {
        requestRoutines[irq] = reinterpret_cast<void*>(handler);
}

void InterruptRequests::remapRequests() {
        outportByte(0x20, 0x11);
        outportByte(0xA0, 0x11);
        outportByte(0x21, 0x20);
        outportByte(0xA1, 0x28);
        outportByte(0x21, 0x04);
        outportByte(0xA1, 0x02);
        outportByte(0x21, 0x01);
        outportByte(0xA1, 0x01);
        outportByte(0x21, 0x0);
        outportByte(0xA1, 0x0);
}

void InterruptRequests::install(Idt &idt) {
		remapRequests();

        idt.createIdtGate(32, (unsigned) _ireq0, 0x08, 0x8E);
        idt.createIdtGate(33, (unsigned) _ireq1, 0x08, 0x8E);
        idt.createIdtGate(34, (unsigned) _ireq2, 0x08, 0x8E);
        idt.createIdtGate(35, (unsigned) _ireq3, 0x08, 0x8E);
        idt.createIdtGate(36, (unsigned) _ireq4, 0x08, 0x8E);
        idt.createIdtGate(37, (unsigned) _ireq5, 0x08, 0x8E);
        idt.createIdtGate(38, (unsigned) _ireq6, 0x08, 0x8E);
        idt.createIdtGate(39, (unsigned) _ireq7, 0x08, 0x8E);
        idt.createIdtGate(40, (unsigned) _ireq8, 0x08, 0x8E);
        // This line crashes ->
        idt.createIdtGate(41, (unsigned) _ireq9, 0x08, 0x8E);
        idt.createIdtGate(42, (unsigned) _ireq10, 0x08, 0x8E);
        idt.createIdtGate(43, (unsigned) _ireq11, 0x08, 0x8E);
        idt.createIdtGate(44, (unsigned) _ireq12, 0x08, 0x8E);
        idt.createIdtGate(45, (unsigned) _ireq13, 0x08, 0x8E);
        idt.createIdtGate(46, (unsigned) _ireq14, 0x08, 0x8E);
        idt.createIdtGate(47, (unsigned) _ireq15, 0x08, 0x8E);
}
